package com.fluxloop.pinch

import com.fluxloop.pinch.sdk.Pinch
import com.fluxloop.pinch.common.model.DemographicProfile
import com.fluxloop.pinch.sdk.PinchMetrics
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

class PinchPlugin: MethodCallHandler {
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "com.fluxloop.pinch/sdk")
      channel.setMethodCallHandler(PinchPlugin())
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    try {
      when (call.method) {
        "start" -> {
          Pinch.start()
          result.success(true)
        }
        "stop" -> {
          Pinch.stop()
          result.success(true)
        }
        "setMessagingId" -> {
          val messagingId: String? = call.argument("messagingId")
          Pinch.messagingId = messagingId
          result.success(true)
        }
        "addCustomEvent" -> {
          val type: String? = call.argument("type")
          val json: String? = call.argument("json")
          if (type == null || json == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            Pinch.addCustomEvent(type, json)
            result.success(true)
          }
        }
        "setMetadata" -> {
          val type: String? = call.argument("type")
          val json: String? = call.argument("json")
          if (type == null || json == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            Pinch.setMetadata(type, json)
            result.success(true)
          }
        }
        "sendDemographicProfile" -> {
          val birthYear: Int? = call.argument("birthYear")
          val gender: String? = call.argument("gender")
          if (birthYear == null || gender == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            val finalGender = when(gender) {
              "male" -> DemographicProfile.Gender.MALE
              "female" -> DemographicProfile.Gender.FEMALE
              "unknown" -> DemographicProfile.Gender.UNKNOWN
              else -> DemographicProfile.Gender.UNKNOWN
            }
            Pinch.sendDemographicProfile(DemographicProfile(birthYear = birthYear, gender = finalGender))
            result.success(true)
          }
        }
        "getStarted" -> {
          result.success(Pinch.started)
        }
        "metric" -> {
          sendMetric(call, result)
        }
        else -> result.notImplemented()
      }
    } catch (e: Exception) {
      result.error("Exception encountered", call.method, e)
    }
  }

  private fun sendMetric(call: MethodCall, result: Result) {
    var failed = false
    val type: Int? = call.argument("type")
    val action: Int? = call.argument("action")

    if (type == null || action == null) {
      result.success(false)
      return
    }

    // 1 == Onboarding
    val version: String? = call.argument("version")
    val requestType: Int? = call.argument("requestType")
    val accepted:  Boolean? = call.argument("accepted")

    if (type == 1) {
      when (action) {
        1 -> PinchMetrics.Onboarding.notified(version = version ?: 0)
        2 -> PinchMetrics.Onboarding.notificationDismissed()
        3 -> PinchMetrics.Onboarding.started(version = version ?: 0)
        4, 5 -> {
          if (requestType == null || accepted == null) {
            failed = true
            return
          } else {
            val requestedType = getRequestType(requestType)
            if (requestedType != null) {
              if (type == 4) {
                PinchMetrics.Onboarding.requestedPermission(requestedType, accepted)
              } else if (type == 5) {
                PinchMetrics.Onboarding.requestedRuntimePermission(requestedType, accepted)
              }
            } else {
              failed = true
            }
          }
        }
        6 -> PinchMetrics.Onboarding.cancelled()
        7 -> PinchMetrics.Onboarding.completed()
        else -> failed = true
      }
    }
    result.success(!failed)
  }

  private fun getRequestType(requestType: Int): PinchMetrics.Onboarding.RequestType? {
    return when (requestType) {
      1 -> PinchMetrics.Onboarding.RequestType.LOCATION
      2 -> PinchMetrics.Onboarding.RequestType.WIFI
      3 -> PinchMetrics.Onboarding.RequestType.BLUETOOTH
      4 -> PinchMetrics.Onboarding.RequestType.MOTION
      6 -> PinchMetrics.Onboarding.RequestType.SURVEYS
      7 -> PinchMetrics.Onboarding.RequestType.ANALYTICS
      8 -> PinchMetrics.Onboarding.RequestType.CAMPAIGNS
      9 -> PinchMetrics.Onboarding.RequestType.ADS
      10 -> PinchMetrics.Onboarding.RequestType.BIRTH_YEAR
      11 -> PinchMetrics.Onboarding.RequestType.GENDER
      12 -> PinchMetrics.Onboarding.RequestType.PINCH
      else -> null
    }
  }
}
